"""
distutilazy.tests
-----------------

Tests for distutilazy

:license: MIT, see LICENSE for more details.
"""

from os.path import basename, splitext, join, dirname
from glob import glob

test_modules = [
    splitext(basename(filename))[0] for filename in glob(
        join(dirname(__file__), 'test*.py'))
    ]
test_modules.sort()

__all__ = test_modules


"""
distutilazy.command.pyinstaller
-------------------------------

Command to call pyinstaller

:license: MIT, see LICENSE for more details.
"""

import distutilazy.pyinstaller


class bdist_pyinstaller(distutilazy.pyinstaller.BdistPyInstaller):
    pass



"""
distutilazy.pyinstaller
-----------------------

command classes to call pyinstaller

:license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import os
import platform
import subprocess
from distutils.core import Command
from distutils.errors import DistutilsOptionError

from . import clean

__version__ = "0.3.0"

is_windows = platform.system().upper() == "WINDOWS"
path_separator = is_windows and ';' or ':'


class BdistPyInstaller(Command):
    """Distutils command to run PyInstaller with configured defaults"""

    description = "Run PyInstaller to create standalone executable programs"

    user_options = [
        ("target=", 't', "Target Python app to bundle"),
        ("pyinstaller-path=", None, "Path to PyInstaller executable"),
        ("name=", 'n', "Name of the bundled app"),
        ("icon=", 'i', "Path to icon resource"),
        ("windowed", 'w', "Windowed app, no console for stdio"),
        ("clean", None, "Clean cached and temp files before build"),
        ("hidden-imports=", 'I',
            "comma separated list of extra modules to be imported"),
        ("paths=", 'p',
         "extra paths to search for modules separated by '{}'".format(
             path_separator)
         ),
    ]

    boolean_options = ["windowed", "clean"]

    def initialize_options(self):
        self.pyinstaller_opts = []
        self.imports = []
        self.syspaths = []
        self.target = None
        self.pyinstaller_path = None
        self.name = None
        self.icon = None
        self.windowed = None
        self.clean = None
        self.hidden_imports = None
        self.paths = None

    def default_pyinstaller_opts(self):
        """Return default options for PyInstaller.
        Use this method to customize the command for separate projects

        :return: list of options
        """
        return ["--onefile"]

    def default_imports(self):
        """Return list of explicit imports.
        Use this method to customize the extra modules for separate projects

        :return: list of module names to be imported
        """
        return []

    def default_paths(self):
        """Return list of paths to append to sys.path while importing modules.
        Use this method to customize the extra paths for separate projects

        :return: list of paths
        """
        return []

    def finalize_options(self):
        if self.pyinstaller_path:
            self.pyinstaller_path = os.path.abspath(self.pyinstaller_path)
            if not os.path.exists(self.pyinstaller_path):
                raise DistutilsOptionError(
                        "failed to find pyinstaller from '{}'".format(
                            self.pyinstaller_path)
                    )
        self.pyinstaller_opts.extend(self.default_pyinstaller_opts())
        if not self.name:
            self.name = self.distribution.metadata.get_name()
        if self.clean:
            self.pyinstaller_opts.append("--clean")
        if self.windowed:
            self.pyinstaller_opts.append("--windowed")
        if self.icon:
            self.pyinstaller_opts.append("--icon=" + self.icon)
        if not is_windows:
            self.pyinstaller_opts.append("--strip")

        self.imports.extend(self.default_imports())
        if self.hidden_imports:
            self.imports.extend(
                [i.strip() for i in self.hidden_imports.split(',') \
                 if i.strip()]
            )
        for module in self.imports:
            self.pyinstaller_opts.append("--hidden-import=" + module)

        self.syspaths.extend(self.default_paths())
        if self.paths:
            self.syspaths.extend(
                [p for p in self.paths.split(path_separator)]
            )
        for path in self.syspaths:
            self.pyinstaller_opts.append("--paths=" + path)
        self.pyinstaller_opts.append("--name=" + self.name)

    def run(self):
        if not self.target:
            raise DistutilsOptionError("no target app is specified to bundle")
        py_installer = self.pyinstaller_path or "pyinstaller"
        args = self.pyinstaller_opts
        args.append(self.target)
        args.insert(0, py_installer)
        self.announce("running " + ' '.join(args))
        code = subprocess.call(args)
        return code


class CleanAll(clean.CleanAll):
    """Distutils command to clean all temporary files,
    compiled Python files, PyInstaller temp files and spec.
    """

    user_options = [
        ("keep-build", None, "do not clean build directory"),
        ("keep-dist", None, "do not clean dist directory"),
        ("keep-egginfo", None, "do not clean egg info directory"),
        ("keep-extra", None, "do not clean extra files"),
        ("name", None, "name of the bundled app"),
    ]

    def initialize_options(self):
        clean.CleanAll.initialize_options(self)
        self.name = None

    def finalize_options(self):
        clean.clean_all.finalize_options(self)
        if not self.name:
            self.name = self.distribution.metadata.get_name()

    def get_extra_paths(self):
        """Return list of extra files/directories to be removed"""
        return [self.name + ".spec"]


bdist_pyinstaller = BdistPyInstaller
clean_all = CleanAll


"""
distutilazy.command.test
------------------------

Command to run unit tests

:license: MIT, see LICENSE for more details.
"""

import distutilazy.test


class test(distutilazy.test.RunTests):
    pass



"""
distutilazy.tests.test_util
-----------------

Test distutilazy.util module.
"""

from __future__ import absolute_import

import sys
from os.path import dirname, realpath
from unittest import TestCase, main

here = dirname(__file__)
sys.path.insert(0, dirname(here))
sys.path.insert(0, here)

from distutilazy import util

if __file__.endswith('$py.class'):
    __file__ = __file__[:-9] + '.py'
elif __file__.endswith('.pyc'):
    __file__ = __file__[:-1]

class TestUtil(TestCase):

    def test_util_find_files(self):
        me = realpath(__file__)
        files = util.find_files(here, "test_util.py*")
        self.assertIn(me, files)
        files = util.find_files(here, "not_existing_file.py")
        self.assertEqual(files, [])

    def test_util_find_directories(self):
        found = util.find_directories(dirname(here), "tes*")
        self.assertIn(here, found)
        found = util.find_directories(here, "not_existing_dir")
        self.assertEqual(found, [])



"""
distutilazy.command
-------------------

Extra distutils commands using command classes provided by distutilazy

:license: MIT, see LICENSE for more details.
"""

__all__ = ("clean_pyc", "clean_all", "clean_jython_class",
           "bdist_pyinstaller", "test")



"""
distutilazy.tests.test_clean
----------------------------

Tests for distutilazy.clean module
"""

from __future__ import absolute_import

import sys
from shutil import rmtree
from os import path, mkdir
from os.path import dirname, abspath
from unittest import TestCase, main
from distutils.dist import Distribution
from tempfile import mkstemp, mkdtemp

here = dirname(__file__)
sys.path.insert(0, dirname(here))
sys.path.insert(0, here)

from distutilazy.clean import CleanPyc, CleanAll, CleanJythonClass


class TestCleanPyc(TestCase):

    @classmethod
    def setUpClass(cls):
        cls.test_cache_dir = abspath(path.join(here, '_test_py_cache_'))
        if path.exists(cls.test_cache_dir):
            raise Exception(
                    "Test python cache directory exists in '{}'." \
                    " Please remove this path".format(cls.test_cache_dir)
                    )
        mkdir(cls.test_cache_dir)

    @classmethod
    def tearDownAfter(cls):
        if path.exists(cls.test_cache_dir):
            rmtree(cls.test_cache_dir, True)

    def test_clean_pyc(self):
        try:
            temp_files = []
            temp_dir = mkdtemp(suffix='_distutilazy_test')
            for i in range(5):
                _, temp_filename = mkstemp(suffix=".pyc", dir=temp_dir)
                temp_files.append(temp_filename)
            temp_files.sort()
            dist = Distribution()
            pyc_cleaner = CleanPyc(dist)
            pyc_cleaner.root = temp_dir
            pyc_cleaner.finalize_options()
            self.assertEqual(
                    sorted(pyc_cleaner.find_compiled_files()),
                    temp_files
            )
            pyc_cleaner.run()

            for temp_filename in temp_files:
                self.assertFalse(path.exists(temp_filename))
        finally:
            if path.exists(temp_dir):
                rmtree(temp_dir, True)

    def test_clean_pyc_finds_nothing(self):
        dist = Distribution()
        pyc_cleaner = CleanPyc(dist)
        pyc_cleaner.extensions = ".ppyycc, .ppyyoo"
        pyc_cleaner.finalize_options()
        self.assertEqual(pyc_cleaner.extensions, [".ppyycc", ".ppyyoo"])
        self.assertEqual(pyc_cleaner.find_compiled_files(), [])

    def test_clean_py_cache_dirs(self):
        dist = Distribution()
        pycache_cleaner = CleanPyc(dist)
        pycache_cleaner.directories = "_test_py_cache_"
        pycache_cleaner.finalize_options()
        self.assertEqual(pycache_cleaner.directories, ["_test_py_cache_"])
        self.assertEqual(
            pycache_cleaner.find_cache_directories(),
            [self.__class__.test_cache_dir]
        )
        pycache_cleaner.run()
        self.assertFalse(path.exists(self.__class__.test_cache_dir))

    def test_clean_py_cache_dirs_finds_nothing(self):
        dist = Distribution()
        pycache_cleaner = CleanPyc(dist)
        pycache_cleaner.extensions = ".ppyycc, .ppyyoo"
        pycache_cleaner.directories = "not_exist, and_not_found"
        pycache_cleaner.finalize_options()
        self.assertEqual(
            pycache_cleaner.directories,
            ["not_exist", "and_not_found"]
        )
        self.assertEqual(pycache_cleaner.find_cache_directories(), [])


class TestCleanAll(TestCase):

    def test_clean_all(self):
        dist = Distribution()
        dist.metadata.name = "test_dist"
        all_cleaner = CleanAll(dist)
        all_cleaner.finalize_options()
        self.assertEqual(all_cleaner.get_egginfo_dir(), "test_dist.egg-info")
        targets = ["build", "dist", "egginfo", "extra"]
        bad_calls = []
        good_calls = []
        good_calls_should_be = 0
        for target in targets:
            all_cleaner = CleanAll(dist)
            all_cleaner.finalize_options()
            all_cleaner.dry_run = True
            setattr(all_cleaner, "keep_%s" % target, True)
            setattr(all_cleaner, "clean_%s" % target,
                    lambda x: bad_calls.append(target))
            other_targets = [t for t in targets if t != target]
            for other_target in other_targets:
                good_calls_should_be += 1
                setattr(all_cleaner, "clean_%s" % other_target,
                        lambda x=None: good_calls.append(other_target))
            all_cleaner.run()
        self.assertEqual(bad_calls, [])
        self.assertEqual(len(good_calls), good_calls_should_be)


class TestCleanJythonClass(TestCase):

    def test_clean_classes_finds_nothing(self):
        dist = Distribution()
        pyc_cleaner = CleanJythonClass(dist)
        pyc_cleaner.extensions = "$py.cla$$, .clazz"
        pyc_cleaner.finalize_options()
        self.assertEqual(pyc_cleaner.extensions, ["$py.cla$$", ".clazz"])
        self.assertEqual(pyc_cleaner.find_class_files(), [])

    def test_clean_class(self):
        try:
            temp_files = []
            temp_dir = mkdtemp(suffix='_distutilazy_test')
            for i in range(5):
                _, temp_filename = mkstemp(suffix="$py.class", dir=temp_dir)
                temp_files.append(temp_filename)
            temp_files.sort()
            dist = Distribution()
            jython_cleaner = CleanJythonClass(dist)
            jython_cleaner.root = temp_dir
            jython_cleaner.finalize_options()
            self.assertEqual(
                    sorted(jython_cleaner.find_class_files()),
                    temp_files
            )
            jython_cleaner.run()

            for temp_filename in temp_files:
                self.assertFalse(path.exists(temp_filename))
        finally:
            if path.exists(temp_dir):
                rmtree(temp_dir, True)


if __name__ == "__main__":
    main()



"""
distutilazy.clean
-----------------

command classes to help clean temporary files

:license: MIT, see LICENSE for more details.
"""

from __future__ import absolute_import

import os
from shutil import rmtree
from distutils import log
from distutils.core import Command
from distutils.command import clean

from . import util

__version__ = "0.4.0"


class BaseFileSystemCleanerCommand(Command):

    @staticmethod
    def default_extensions(cls):
        return []

    @staticmethod
    def default_directories(cls):
        return []

    def initialize_options(self):
        self.root = os.getcwd()
        self.extensions = ','.join(self.default_extensions())
        self.directories = ','.join(self.default_directories())

    def finalize_options(self):
        if not os.path.exists(self.root):
            raise IOError("Failed to access root path '{}'".format(self.root))
        self.extensions = [ext.strip() for ext in self.extensions.split(',')]
        self.directories = [
            dir_.strip() for dir_ in self.directories.split(',')]

    def _find_files(self):
        """Find files recursively in the root path
        using provided extensions.

        :return: list of absolute file paths
        """
        files = []
        for ext in self.extensions:
            ext_files = util.find_files(self.root, "*" + ext)
            log.debug("found {} '*{}' files in '{}'".format(
                len(ext_files), ext, self.root)
            )
            files.extend(ext_files)
        return files

    def _find_directories(self):
        directories = []
        for dir_name in self.directories:
            dirs = util.find_directories(self.root, dir_name)
            log.debug("found {} directories in '{}'".format(
                len(dirs), self.root))
            directories.extend(dirs)
        return directories

    def _clean_file(self, filename):
        """Clean a file if exists and not in dry run"""
        if not os.path.exists(filename):
            return
        self.announce("removing '{}'".format(filename))
        if not self.dry_run:
            os.remove(filename)

    def _clean_directory(self, name):
        """Clean a directory if exists and not in dry run"""
        if not os.path.exists(name):
            return
        self.announce(
            "removing directory '{}' and all its contents".format(name)
        )
        if not self.dry_run:
            rmtree(name, True)


class CleanPyc(BaseFileSystemCleanerCommand):
    description = """Clean root dir from complied python files"""
    user_options = [("root=", "r", "path to root dir")]

    @staticmethod
    def default_extensions():
        return [".pyc", ".pyo", ".pyd"]

    @staticmethod
    def default_directories():
        return ["__pycache__"]

    def find_compiled_files(self):
        """Find compiled Python files recursively in the root path

        :return: list of absolute file paths
        """
        files = self._find_files()
        self.announce(
            "found '{}' compiled python files in '{}'".format(
                len(files), self.root
            )
        )
        return files

    def find_cache_directories(self):
        directories = self._find_directories()
        self.announce(
            "found {} python cache directories in '{}'".format(
                len(directories), self.root
            )
        )
        return directories

    def run(self):
        directories = self.find_cache_directories()
        if directories:
            self.announce(
                "cleaning python cache directories in '{}' ...".format(
                    self.root))
            if not self.dry_run:
                for dir_name in directories:
                    self._clean_directory(dir_name)

        files = self.find_compiled_files()
        if files:
            self.announce(
                "cleaning compiled python files in '{}' ...".format(self.root))
            if not self.dry_run:
                for filename in files:
                    self._clean_file(filename)


class CleanJythonClass(BaseFileSystemCleanerCommand):
    description = """Clean root dir from complied files created by Jython"""
    user_options = [("root=", "r", "path to root dir")]

    @staticmethod
    def default_extensions():
        return ["$py.class"]

    @staticmethod
    def default_directories():
        return []

    def find_class_files(self):
        """Find compiled class files recursively in the root path

        :return: list of absolute file paths
        """
        files = self._find_files()
        self.announce(
            "found '{}' compiled class files in '{}'".format(
                len(files), self.root
            )
        )
        return files

    def run(self):
        files = self.find_class_files()
        if files:
            self.announce(
                "cleaning compiled class files in '{}' ...".format(self.root))
            if not self.dry_run:
                for filename in files:
                    self._clean_file(filename)


class CleanAll(clean.clean, BaseFileSystemCleanerCommand):
    description = "Clean root dir from temporary files (complied files, etc)"
    user_options = [
        ("keep-build", None, "do not clean build directory"),
        ("keep-dist", None, "do not clean dist directory"),
        ("keep-egginfo", None, "do not clean egg info directory"),
        ("keep-extra", None, "do not clean extra files"),
    ]

    boolean_options = ["keep-build", "keep-dist", "keep-egginfo", "keep-extra"]

    @staticmethod
    def default_extensions():
        return CleanPyc.default_extensions() + \
                CleanJythonClass.default_extensions()

    @staticmethod
    def default_directories():
        return CleanPyc.default_directories() + \
                CleanJythonClass.default_directories()

    def initialize_options(self):
        clean.clean.initialize_options(self)
        BaseFileSystemCleanerCommand.initialize_options(self)
        self.keep_build = None
        self.keep_dist = None
        self.keep_egginfo = None
        self.keep_extra = None

    def finalize_options(self):
        clean.clean.finalize_options(self)
        BaseFileSystemCleanerCommand.finalize_options(self)
        self.all = True

    def get_egginfo_dir(self):
        return self.distribution.metadata.get_name() + ".egg-info"

    def get_extra_paths(self):
        """Return list of extra files/directories to be removed"""
        return []

    def clean_egginfo(self):
        """Clean .egginfo directory"""
        dir_name = os.path.join(self.root, self.get_egginfo_dir())
        self._clean_directory(dir_name)

    def clean_dist(self):
        self._clean_directory(os.path.join(self.root, "dist"))

    def clean_build(self):
        self._clean_directory(os.path.join(self.root, "build"))

    def clean_extra(self):
        """Clean extra files/directories specified by get_extra_paths()"""
        extra_paths = self.get_extra_paths()
        for path in extra_paths:
            if not os.path.exists(path):
                continue
            if os.path.isdir(path):
                self._clean_directory(path)
            else:
                self._clean_file(path)

    def run(self):
        clean.clean.run(self)
        if not self.keep_build:
            self.clean_build()
        if not self.keep_egginfo:
            self.clean_egginfo()
        if not self.keep_dist:
            self.clean_dist()
        if not self.keep_extra:
            self.clean_extra()
        directories = self._find_directories()
        if directories and not self.dry_run:
            for dir_name in directories:
                self._clean_directory(dir_name)

        files = self._find_files()
        if files and not self.dry_run:
            for filename in files:
                self._clean_file(filename)


clean_pyc = CleanPyc
clean_all = CleanAll



#!/usr/bin/env python

"""
distutilazy
-----------

Extra distutils commands.

:license: MIT, see LICENSE for more details.
"""

from __future__ import print_function

import os
import sys

try:
    import setuptools
    from setuptools import setup
except ImportError as exp:
    setuptools = None
    from distutils.core import setup

sys.path.insert(0, os.path.dirname(os.path.abspath(__file__)))

import distutilazy
import distutilazy.clean
import distutilazy.test

CLASSIFIERS = [
    "Development Status :: 4 - Beta",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: MIT License",
    "Operating System :: OS Independent",
    "Programming Language :: Python",
    "Programming Language :: Python :: 2.7",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.2",
    "Programming Language :: Python :: 3.3",
    "Programming Language :: Python :: 3.4",
    "Programming Language :: Python :: 3.5",
    "Programming Language :: Python :: Implementation :: CPython",
    "Programming Language :: Python :: Implementation :: PyPy",
    "Topic :: Software Development :: Libraries :: Python Modules",
    "Topic :: System :: Archiving :: Packaging",
    "Topic :: System :: Systems Administration",
]

long_description = __doc__
with open(os.path.join(os.path.dirname(__file__), "README.rst")) as fh:
    long_description = fh.read()

params = dict(
    name = "distutilazy",
    author = "Farzad Ghanei",
    author_email = "farzad.ghanei@gmail.com",
    url = "http://github.com/farzadghanei/distutilazy/",
    packages = ["distutilazy", "distutilazy.command"],
    version = distutilazy.__version__,
    description = "Extra distutils commands",
    long_description = long_description,
    license = "MIT",
    classifiers = CLASSIFIERS,
    cmdclass = {
        "clean_pyc": distutilazy.clean.CleanPyc,
        "clean_jython_class": distutilazy.clean.CleanJythonClass,
        "clean_all": distutilazy.clean.CleanAll,
        "test": distutilazy.test.RunTests
        }
)

if setuptools:
    params.update(zip_safe = True)

dist = setup(**params)



"""
distutilazy.command.clean_jython_class
--------------------------------------

Command to clean compiled .class files created by Jython.

:license: MIT, see LICENSE for more details.
"""

import distutilazy.clean


class clean_jython_class(distutilazy.clean.CleanJythonClass):
    pass
